"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const path_1 = __importDefault(require("path"));
const postcss_1 = __importDefault(require("postcss"));
const postcss_icss_selectors_1 = __importDefault(require("postcss-icss-selectors"));
const tags_1 = require("./tags");
const sass_1 = __importDefault(require("sass"));
const SIGN = '// cssx';
const processor = (0, postcss_1.default)((0, postcss_icss_selectors_1.default)());
const isStyle = (fileName) => /\.(css|scss|sass)$/.test(fileName);
const isRelative = (fileName) => /^\.\.?($|[\\/])/.test(fileName);
function init({ typescript: ts }) {
    const create = (info) => {
        // prettier-ignore
        const { createLanguageServiceSourceFile } = ts;
        ts.createLanguageServiceSourceFile = (fileName, scriptSnapshot, ...rest) => {
            if (isStyle(fileName)) {
                scriptSnapshot = createDtsSnapshot(ts, scriptSnapshot, fileName);
            }
            const sourceFile = createLanguageServiceSourceFile(fileName, scriptSnapshot, ...rest);
            if (isStyle(fileName)) {
                sourceFile.isDeclarationFile = true;
            }
            return sourceFile;
        };
        const { updateLanguageServiceSourceFile } = ts;
        ts.updateLanguageServiceSourceFile = (sourceFile, scriptSnapshot, ...rest) => {
            const { fileName } = sourceFile;
            if (isStyle(fileName)) {
                scriptSnapshot = createDtsSnapshot(ts, scriptSnapshot, fileName);
            }
            sourceFile = updateLanguageServiceSourceFile(sourceFile, scriptSnapshot, ...rest);
            if (isStyle(fileName)) {
                sourceFile.isDeclarationFile = true;
            }
            return sourceFile;
        };
        if (info.languageServiceHost.resolveModuleNames) {
            const { resolveModuleNames } = info.languageServiceHost;
            info.languageServiceHost.resolveModuleNames = (moduleNames, containingFile, ...rest) => {
                const resolvedModules = resolveModuleNames.call(info.languageServiceHost, moduleNames, containingFile, ...rest);
                return moduleNames.map((moduleName, index) => {
                    if (isStyle(moduleName) && isRelative(moduleName)) {
                        const fileName = path_1.default.resolve(path_1.default.dirname(containingFile), moduleName);
                        return {
                            extension: ts.Extension.Dts,
                            isExternalLibraryImport: false,
                            resolvedFileName: fileName,
                        };
                    }
                    else {
                        return resolvedModules[index];
                    }
                });
            };
        }
        return info.languageService;
    };
    const getExternalFiles = (project) => {
        return project.getFileNames().filter((name) => isStyle(name));
    };
    return { create, getExternalFiles };
}
const createDtsSnapshot = (ts, scriptSnapshot, fileName) => {
    const source = scriptSnapshot.getText(0, scriptSnapshot.getLength());
    if (source.startsWith(SIGN)) {
        return scriptSnapshot;
    }
    const classnames = getClassnames(fileName);
    const dts = [];
    dts.push(SIGN);
    dts.push(`type Props = {`);
    dts.push('[k: `css:$${string}`]: string | number');
    if (classnames.length) {
        for (const classname of classnames) {
            dts.push(`"css:${classname}"?: boolean`);
        }
    }
    else {
        dts.push(`[k: string]?: any`);
    }
    dts.push(`}`);
    dts.push(`type Component<T extends keyof JSX.IntrinsicElements> = (props: JSX.IntrinsicElements[T] & Props) => JSX.Element`);
    for (let i = 0; i < tags_1.tags.length; i++) {
        const tag = tags_1.tags[i];
        const component = tags_1.components[i];
        dts.push(`export const ${component}: Component<'${tag}'>`);
    }
    dts.push(`declare const styles: {`);
    for (const classname of classnames) {
        dts.push(`${classname}: string`);
    }
    dts.push(`}`);
    dts.push(`export default styles`);
    return ts.ScriptSnapshot.fromString(dts.join(`\n`));
};
const getClassnames = (from) => {
    const classnames = [];
    try {
        const { css } = sass_1.default.compile(from);
        const compiled = processor.process(css, { from });
        for (const { name, value } of compiled.messages) {
            classnames.push(name);
        }
    }
    catch (e) {
        console.log(e);
    }
    return classnames;
};
module.exports = init;
